/* KallistiOS 1.1.7

   exemple.c
   Eldrad Uhltran
   eldrad@rpgtown.com
   http://creezvosjeux.online.fr/dreamcast/
*/

#include <kos.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <pcx/pcx.h>


/* Exemple simple d'affichage d'une image
avec effet de fade sous KGL
*/

GLuint texture[1];	/* Stock une Texture */
GLfloat nfade=0.0f;
int fadeout=true;
int fadein=false;
int fin=false;

/* Charge une texture pcx et lance glKosTex2D */
void loadtxr(const char *fn, GLuint *txr) {
	kos_img_t img;
	pvr_ptr_t txaddr;

	if (pcx_to_img(fn, &img) < 0) {
		printf("impossible de charger %s\n", fn);
		return;
	}

	txaddr = pvr_mem_malloc(img.w * img.h * 2);
	pvr_txr_load_kimg(&img, txaddr, PVR_TXRLOAD_INVERT_Y);
	kos_img_free(&img, 0);

	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(GL_RGB565_TWID, img.w, img.h, txaddr);
}

void draw_gl(void) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glTranslatef(0.0f,0.0f,-5.0f);

	/* Principe: on dessine un carr sur lequel on
	plaque la texture de l'image, on le teint de la couleur
	voulue pour lui donner l'effet de fade: si noir, cran devient
	noir, si blanc, l'image est affiche, et entre, images intermdiaires
	(alpha blending), donc effet de fondu*/
	glColor4f(nfade,nfade,nfade, 1.0f);
	glBindTexture(GL_TEXTURE_2D, texture[0]);
	glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 0.0f);glVertex3f(-1.2f, -1.2f,  3.0f);
			glTexCoord2f(1.0f, 0.0f);glVertex3f( 1.2f, -1.2f,  3.0f);
			glTexCoord2f(1.0f, 1.0f);glVertex3f( 1.2f,  1.2f,  3.0f);
			glTexCoord2f(0.0f, 1.0f);glVertex3f(-1.2f,  1.2f,  3.0f);
	glEnd();

	/* Calcul des coefficients de fade*/
	if(fadeout)/*Apparition image*/
	{
		if(nfade>=1.0f){
			fadeout=false;
			nfade=1.0f;
			}
		else
			nfade+=0.01f;
	}

	if(fadein)/*Disparition image*/
	{
		if(nfade<=0.0f)
			fin=true;
		else
			nfade-=0.01f;
	}
}

extern uint8 romdisk[];
KOS_INIT_ROMDISK(romdisk);

int main(int argc, char **argv) {
	cont_cond_t cond;
	uint8	c;

	/* Initialisation KOS */
	pvr_init_defaults();

	/* Initialisations KGL */
	glKosInit();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,640.0f/480.0f,0.1f,100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_TEXTURE_2D);
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);

	/* charge la texture */
	loadtxr("/rd/titre.pcx", &texture[0]);

	c = maple_first_controller();
	while(!fin) {
		/* on vrifie les boutons de la manette */
		if (cont_get_cond(c, &cond) < 0) {
			printf("Impossible de lire la manette\n");
			break;
		}
		if (!(cond.buttons & CONT_START))
			fadein=true;

		/* dbut rendu image */
		glKosBeginFrame();

		/* Dessine la scne */
		draw_gl();

		/* fin rendu image */
		glKosFinishFrame();
	}

	return 0;
}


